/**
 * DO NOT EDIT
 *
 * This file was automatically generated by
 *   https://github.com/Polymer/tools/tree/master/packages/gen-typescript-declarations
 *
 * To modify these typings, edit the source file(s):
 *   web_modules/jexl/lib/Jexl.js
 */


// tslint:disable:variable-name Describing an API that's defined elsewhere.
// tslint:disable:no-any describes the API as best we are able today

/**
 * The Evaluator takes a Jexl expression tree as generated by the
 * {@link Parser} and calculates its value within a given context. The
 * collection of transforms, context, and a relative context to be used as the
 * root for relative identifiers, are all specific to an Evaluator instance.
 * When any of these things change, a new instance is required.  However, a
 * single instance can be used to simultaneously evaluate many different
 * expressions, and does not have to be reinstantiated for each.
 */
declare class Evaluator {
  constructor(grammar: any, transforms: any, context: any, relativeContext: any, promise?: any);

  /**
   * Evaluates an expression tree within the configured context.
   *
   * @param ast An expression tree object
   * @returns resolves with the resulting value of the expression.
   */
  eval(ast: {}): Promise<any>;

  /**
   * Simultaneously evaluates each expression within an array, and delivers the
   * response as an array with the resulting values at the same indexes as their
   * originating expressions.
   *
   * @param arr An array of expression strings to be evaluated
   * @returns resolves with the result array
   */
  evalArray(arr: string[]|null): Promise<Array<{}>|null>;

  /**
   * Simultaneously evaluates each expression within a map, and delivers the
   * response as a map with the same keys, but with the evaluated result for each
   * as their value.
   *
   * @param map A map of expression names to expression trees to be
   *      evaluated
   * @returns resolves with the result map.
   */
  evalMap(map: {}): Promise<{}>;
}

/**
 * Lexer is a collection of stateless, statically-accessed functions for the
 * lexical parsing of a Jexl string.  Its responsibility is to identify the
 * "parts of speech" of a Jexl expression, and tokenize and label each, but
 * to do only the most minimal syntax checking; the only errors the Lexer
 * should be concerned with are if it's unable to identify the utility of
 * any of its tokens.  Errors stemming from these tokens not being in a
 * sensible configuration should be left for the Parser to handle.
 */
declare class Lexer {
  constructor(grammar: any);

  /**
   * Splits a Jexl expression string into an array of expression elements.
   *
   * @param str A Jexl expression string
   * @returns An array of substrings defining the functional
   *      elements of the expression.
   */
  getElements(str: string): string[]|null;

  /**
   * Converts an array of expression elements into an array of tokens.  Note that
   * the resulting array may not equal the element array in length, as any
   * elements that consist only of whitespace get appended to the previous
   * token's "raw" property.  For the structure of a token object, please see
   * {@link Lexer#tokenize}.
   *
   * @param elements An array of Jexl expression elements to be
   *      converted to tokens
   * @returns an array of token objects.
   */
  getTokens(elements: string[]|null): Array<{type: any, value: any, raw: any}>|null;

  /**
   * Converts a Jexl string into an array of tokens.  Each token is an object
   * in the following format:
   *
   *     {
   *         type: <string>,
   *         [name]: <string>,
   *         value: <boolean|number|string>,
   *         raw: <string>
   *     }
   *
   * Type is one of the following:
   *
   *      literal, identifier, binaryOp, unaryOp
   *
   * OR, if the token is a control character its type is the name of the element
   * defined in the Grammar.
   *
   * Name appears only if the token is a control string found in
   * {@link grammar#elements}, and is set to the name of the element.
   *
   * Value is the value of the token in the correct type (boolean or numeric as
   * appropriate). Raw is the string representation of this value taken directly
   * from the expression string, including any trailing spaces.
   *
   * @param str The Jexl string to be tokenized
   * @returns an array of token objects.
   */
  tokenize(str: string): Array<{type: any, value: any, raw: any}>|null;

  /**
   * Creates a new token object from an element of a Jexl string. See
   * {@link Lexer#tokenize} for a description of the token object.
   *
   * @param element The element from which a token should be made
   */
  _createToken(element: string): any;
}

/**
 * The Parser is a state machine that converts tokens from the {@link Lexer}
 * into an Abstract Syntax Tree (AST), capable of being evaluated in any
 * context by the {@link Evaluator}.  The Parser expects that all tokens
 * provided to it are legal and typed properly according to the grammar, but
 * accepts that the tokens may still be in an invalid order or in some other
 * unparsable configuration that requires it to throw an Error.
 */
declare class Parser {
  constructor(grammar: any, prefix: any, stopMap: any);

  /**
   * Processes a new token into the AST and manages the transitions of the state
   * machine.
   */
  addToken(token: any): any;

  /**
   * Processes an array of tokens iteratively through the {@link #addToken}
   * function.
   */
  addTokens(tokens: any): void;

  /**
   * Marks this Parser instance as completed and retrieves the full AST.
   *
   * @returns a full expression tree, ready for evaluation by the
   *      {@link Evaluator#eval} function, or null if no tokens were passed to
   *      the parser before complete was called
   */
  complete(): {}|null;

  /**
   * Indicates whether the expression tree contains a relative path identifier.
   *
   * @returns true if a relative identifier exists false otherwise.
   */
  isRelative(): boolean;

  /**
   * Places a new tree node at the current position of the cursor (to the 'right'
   * property) and then advances the cursor to the new node. This function also
   * handles setting the parent of the new node.
   */
  _placeAtCursor(node: any): void;

  /**
   * Places a tree node before the current position of the cursor, replacing
   * the node that the cursor currently points to. This should only be called in
   * cases where the cursor is known to exist, and the provided node already
   * contains a pointer to what's at the cursor currently.
   */
  _placeBeforeCursor(node: any): void;

  /**
   * Sets the parent of a node by creating a non-enumerable _parent property
   * that points to the supplied parent argument.
   */
  _setParent(node: any, parent: any): void;

  /**
   * Prepares the Parser to accept a subexpression by (re)instantiating the
   * subParser.
   */
  _startSubExpression(exprStr: any): void;
}

declare class PromiseSync {
  constructor(fn: any);
  catch(rejected: any): any;
  then(resolved: any, rejected: any): any;
  _reject(error: any): void;
  _resolve(val: any): void;
}

/**
 * Jexl is the Javascript Expression Language, capable of parsing and
 * evaluating basic to complex expression strings, combined with advanced
 * xpath-like drilldown into native Javascript objects.
 */
declare class Jexl {
  constructor();

  /**
   * Adds a binary operator to Jexl at the specified precedence. The higher the
   * precedence, the earlier the operator is applied in the order of operations.
   * For example, * has a higher precedence than +, because multiplication comes
   * before division.
   *
   * Please see grammar.js for a listing of all default operators and their
   * precedence values in order to choose the appropriate precedence for the
   * new operator.
   *
   * @param operator The operator string to be added
   * @param precedence The operator's precedence
   * @param fn A function to run to calculate the result. The function
   *      will be called with two arguments: left and right, denoting the values
   *      on either side of the operator. It should return either the resulting
   *      value, or a Promise that resolves with the resulting value.
   */
  addBinaryOp(operator: string, precedence: number, fn: function|null): void;

  /**
   * Adds a unary operator to Jexl. Unary operators are currently only supported
   * on the left side of the value on which it will operate.
   *
   * @param operator The operator string to be added
   * @param fn A function to run to calculate the result. The function
   *      will be called with one argument: the literal value to the right of the
   *      operator. It should return either the resulting value, or a Promise
   *      that resolves with the resulting value.
   */
  addUnaryOp(operator: string, fn: function|null): void;

  /**
   * Adds or replaces a transform function in this Jexl instance.
   *
   * @param name The name of the transform function, as it will be used
   *      within Jexl expressions
   * @param fn The function to be executed when this transform is
   *      invoked. It will be provided with at least one argument:
   *          - {*} value: The value to be transformed
   *          - {...*} args: The arguments for this transform
   */
  addTransform(name: string, fn: function|null): void;

  /**
   * Syntactic sugar for calling {@link #addTransform} repeatedly.  This function
   * accepts a map of one or more transform names to their transform function.
   *
   * @param map A map of transform names to transform functions
   */
  addTransforms(map: {}): void;

  /**
   * Retrieves a previously set transform function.
   *
   * @param name The name of the transform function
   * @returns The transform function
   */
  getTransform(name: string): function|null;

  /**
   * Asynchronously evaluates a Jexl string within an optional context.
   *
   * @param expression The Jexl expression to be evaluated
   */
  eval(expression: string, context?: any): any;

  /**
   * Synchronously evaluates a Jexl string within an optional context.
   *
   * @param expression The Jexl expression to be evaluated
   */
  evalSync(expression: string, context?: any): any;

  /**
   * Removes a binary or unary operator from the Jexl grammar.
   *
   * @param operator The operator string to be removed
   */
  removeOp(operator: string): void;

  /**
   * Adds an element to the grammar map used by this Jexl instance, cloning
   * the default grammar first if necessary.
   *
   * @param str The key string to be added
   */
  _addGrammarElement(str: string, obj: any): void;

  /**
   * Evaluates a Jexl string within an optional context.
   *
   * @param expression The Jexl expression to be evaluated
   */
  _eval(expression: string, context: any, promise: any): any;
}

export {Jexl_2 as Jexl};
